import { oneLine } from 'common-tags';
export function ensureCleanLocalGitStateFactory(dependencies, options) {
    const { gitCommandRunner, findRemoteAlias } = dependencies;
    async function ensureCleanLocalCopy() {
        const status = await gitCommandRunner.getShortStatus();
        if (status !== '') {
            throw new Error('Local copy is not clean');
        }
    }
    async function ensureDefaultBranch() {
        const branchName = await gitCommandRunner.getCurrentBranchName();
        if (branchName !== options.defaultBranch) {
            throw new Error(`Not on ${options.defaultBranch} branch`);
        }
    }
    async function ensureLocalIsEqualToRemote(remoteAlias) {
        const remoteBranch = `${remoteAlias}/${options.defaultBranch}`;
        const commits = await gitCommandRunner.getSymmetricDifferencesBetweenBranches(options.defaultBranch, remoteBranch);
        let commitsAhead = 0;
        let commitsBehind = 0;
        commits.forEach((commit) => {
            if (commit.startsWith('>')) {
                commitsBehind += 1;
            }
            else {
                commitsAhead += 1;
            }
        });
        if (commitsAhead > 0 || commitsBehind > 0) {
            const errorMessage = oneLine `Local git ${options.defaultBranch} branch is ${commitsAhead} commits ahead
                and ${commitsBehind} commits behind of ${remoteBranch}`;
            throw new Error(errorMessage);
        }
    }
    return async function ensureCleanLocalGitState(githubRepo) {
        await ensureCleanLocalCopy();
        await ensureDefaultBranch();
        const remoteAlias = await findRemoteAlias(githubRepo);
        await gitCommandRunner.fetchRemote(remoteAlias);
        await ensureLocalIsEqualToRemote(remoteAlias);
    };
}
//# sourceMappingURL=ensure-clean-local-git-state.js.map