import parseGitUrl from 'git-url-parse';
function isSameGitUrl(gitUrlA, gitUrlB) {
    const parsedUrlA = parseGitUrl(gitUrlA);
    const parsedUrlB = parseGitUrl(gitUrlB);
    const pathA = parsedUrlA.pathname.replace(/\.git$/, '');
    const pathB = parsedUrlB.pathname.replace(/\.git$/, '');
    return parsedUrlA.resource === parsedUrlB.resource && pathA === pathB;
}
function getGitUrl(githubRepo) {
    return `git://github.com/${githubRepo}.git`;
}
export function findRemoteAliasFactory(dependencies) {
    const { gitCommandRunner } = dependencies;
    return async function findRemoteAlias(githubRepo) {
        const gitRemote = getGitUrl(githubRepo);
        const remotes = await gitCommandRunner.getRemoteAliases();
        const matchedRemote = remotes.find((remote) => {
            return isSameGitUrl(gitRemote, remote.url);
        });
        if (matchedRemote === undefined) {
            throw new TypeError(`This local git repository doesn’t have a remote pointing to ${gitRemote}`);
        }
        return matchedRemote.alias;
    };
}
//# sourceMappingURL=find-remote-alias.js.map