import { splitByString } from './split.js';
function determineRepoDetails(githubRepo) {
    const [owner, repo] = splitByString(githubRepo, '/');
    if (repo === undefined) {
        throw new TypeError('Could not find a repository');
    }
    return [owner, repo];
}
async function fetchLabels(githubClient, githubRepo, pullRequestId) {
    const [owner, repo] = determineRepoDetails(githubRepo);
    const params = { owner, repo, issue_number: pullRequestId };
    const { data: labels } = await githubClient.issues.listLabelsOnIssue(params);
    return labels;
}
export async function getPullRequestLabel(githubRepo, validLabels, pullRequestId, dependencies) {
    const { githubClient } = dependencies;
    const validLabelNames = Array.from(validLabels.keys());
    const labels = await fetchLabels(githubClient, githubRepo, pullRequestId);
    const listOfLabels = validLabelNames.join(', ');
    const filteredLabels = labels.filter((label) => {
        return validLabelNames.includes(label.name);
    });
    const [firstLabel] = filteredLabels;
    if (filteredLabels.length > 1) {
        throw new Error(`Pull Request #${pullRequestId} has multiple labels of ${listOfLabels}`);
    }
    else if (firstLabel === undefined) {
        throw new TypeError(`Pull Request #${pullRequestId} has no label of ${listOfLabels}`);
    }
    return firstLabel.name;
}
//# sourceMappingURL=get-pull-request-label.js.map